@@ -19,6 +19,9 @@ my %movMap = (
     XMP       => 'UserData',    # MOV-Movie-UserData-XMP
     UserData  => 'Movie',       # MOV-Movie-UserData
     Movie     => 'MOV',
+    GSpherical => 'SphericalVideoXML', # MOV-Movie-Track-SphericalVideoXML
+    SphericalVideoXML => 'Track',      # (video track specifically, don't create if it doesn't exist)
+    Track     => 'Movie',
 );
 my %mp4Map = (
     # MP4 ('ftyp' compatible brand 'mp41', 'mp42' or 'f4v ') -> XMP at top level
@@ -29,6 +32,9 @@ my %mp4Map = (
     UserData  => 'Movie',       # MOV-Movie-UserData
     Movie     => 'MOV',
     XMP       => 'MOV',         # MOV-XMP
+    GSpherical => 'SphericalVideoXML', # MOV-Movie-Track-SphericalVideoXML
+    SphericalVideoXML => 'Track',      # (video track specifically, don't create if it doesn't exist)
+    Track     => 'Movie',
 );
 my %heicMap = (
     # HEIC ('ftyp' compatible brand 'heic' or 'mif1') -> XMP/EXIF in top level 'meta'
@@ -691,7 +697,7 @@ sub WriteQuickTime($$$)
     my ($et, $dirInfo, $tagTablePtr) = @_;
     $et or return 1;    # allow dummy access to autoload this package
     my ($mdat, @mdat, @mdatEdit, $edit, $track, $outBuff, $co, $term, $delCount);
-    my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err);
+    my (%langTags, $canCreate, $delGrp, %boxPos, %didDir, $writeLast, $err, $atomCount);
     my $outfile = $$dirInfo{OutFile} || return 0;
     my $raf = $$dirInfo{RAF};       # (will be null for lower-level atoms)
     my $dataPt = $$dirInfo{DataPt}; # (will be null for top-level atoms)
@@ -762,7 +768,14 @@ sub WriteQuickTime($$$)
         $canCreate = 1;
         $delGrp = $$et{DEL_GROUP}{$dirName};
     }
+    $atomCount = $$tagTablePtr{VARS}{ATOM_COUNT} if $$tagTablePtr{VARS};
+
     for (;;) {      # loop through all atoms at this level
+        if (defined $atomCount and --$atomCount < 0 and $dataPt) {
+            # stop processing now and just copy the rest of the atom
+            Write($outfile, substr($$dataPt, $raf->Tell())) or $rtnVal=$rtnErr, $err=1;
+            last;
+        }
         my ($hdr, $buff, $keysIndex);
         my $n = $raf->Read($hdr, 8);
         unless ($n == 8) {
@@ -795,6 +808,7 @@ sub WriteQuickTime($$$)
             $size < 0 and $et->Error('Invalid extended atom size'), last;
         } elsif ($size == -8) {
             if ($dataPt) {
+                last if $$dirInfo{DirName} eq 'CanonCNTH';  # (this is normal for Canon CNTH atom)
                 my $pos = $raf->Tell() - 4;
                 $raf->Seek(0,2);
                 my $str = $$dirInfo{DirName} . ' with ' . ($raf->Tell() - $pos) . ' bytes';
@@ -807,15 +821,8 @@ sub WriteQuickTime($$$)
             }
             last;
         } elsif ($size < 0) {
-            if ($$tagTablePtr{VARS}{IGNORE_BAD_ATOMS} and $dataPt) {
-                # ignore bad atom and just copy the rest of this directory
-                $buff = substr($$dataPt, $raf->Tell());
-                Write($outfile, $hdr, $buff) or $rtnVal=$rtnErr, $err=1;
-                last;
-            } else {
-                $et->Error('Invalid atom size');
-                last;
-            }
+            $et->Error('Invalid atom size');
+            last;
         }
 
         # keep track of 'mdat' atom locations for writing later
@@ -847,7 +854,12 @@ sub WriteQuickTime($$$)
             $et->Error("Truncated $tag atom");
             return $rtnVal;
         }
-
+        # save the handler type for this track
+        if ($tag eq 'hdlr' and length $buff >= 12) {
+            my $hdlr = substr($buff,8,4);
+            $$et{HandlerType} = $hdlr if $hdlr =~ /^(vide|soun)$/;
+        }
+        
         # if this atom stores offsets, save its location so we can fix up offsets later
         # (are there any other atoms that may store absolute file offsets?)
         if ($tag =~ /^(stco|co64|iloc|mfra|moof|sidx|saio|gps |CTBO|uuid)$/) {
@@ -937,6 +949,8 @@ sub WriteQuickTime($$$)
 
             if ($subdir) {  # process atoms in this container from a buffer in memory
 
+                undef $$et{HandlerType} if $tag eq 'trak';  # init handler type for this track
+
                 my $subName = $$subdir{DirName} || $$tagInfo{Name};
                 my $start = $$subdir{Start} || 0;
                 my $base = ($$dirInfo{Base} || 0) + $raf->Tell() - $size;
@@ -964,6 +978,7 @@ sub WriteQuickTime($$$)
                     Multi    => $$subdir{Multi},    # necessary?
                     OutFile  => $outfile,
                     NoRefTest=> 1,     # don't check directory references
+                    WriteGroup => $$tagInfo{WriteGroup},
                     # initialize array to hold details about chunk offset table
                     # (each entry has 3-5 items: 0=atom type, 1=table offset, 2=table size,
                     #  3=optional base offset, 4=optional item ID)
@@ -1228,11 +1243,11 @@ sub WriteQuickTime($$$)
         } elsif ($tag eq 'stsd' and length($buff) >= 8) {
             my $n = Get32u(\$buff, 4);      # get number of sample descriptions in table
             my ($pos, $flg) = (8, 0);
-            my $i;
+            my ($i, $msg);
             for ($i=0; $i<$n; ++$i) {       # loop through sample descriptions
-                last if $pos + 16 > length($buff);
+                $pos + 16 <= length($buff) or $msg = 'Truncated sample table', last;
                 my $siz = Get32u(\$buff, $pos);
-                last if $pos + $siz > length($buff);
+                $pos + $siz <= length($buff) or $msg = 'Truncated sample table', last;
                 my $drefIdx = Get16u(\$buff, $pos + 14);
                 my $drefTbl = $$et{QtDataRef};
                 if (not $drefIdx) {
@@ -1242,12 +1257,16 @@ sub WriteQuickTime($$$)
                     # $flg = 0x01-in this file, 0x02-in some other file
                     $flg |= ($$dref[1] == 1 and $$dref[0] ne 'rsrc') ? 0x01 : 0x02;
                 } else {
-                    my $grp = $$et{CUR_WRITE_GROUP} || $parent;
-                    $et->Error("No data reference for $grp sample description $i");
-                    return $rtnVal;
+                    $msg = "No data reference for sample description $i";
+                    last;
                 }
                 $pos += $siz;
             }
+            if ($msg) {
+                my $grp = $$et{CUR_WRITE_GROUP} || $parent;
+                $et->Error("$msg for $grp");
+                return $rtnErr;
+            }
             $$et{QtDataFlg} = $flg;
         }
         if ($tagInfo and $$tagInfo{WriteLast}) {
@@ -1301,6 +1320,9 @@ sub WriteQuickTime($$$)
         # (note that $tag may be a binary Keys index here)
         foreach $tag (@addTags) {
             my $tagInfo = $$dirs{$tag} || $$newTags{$tag};
+            next if defined $$tagInfo{CanCreate} and not $$tagInfo{CanCreate};
+            next if defined $$tagInfo{HandlerType} and
+                (not $$et{HandlerType} or $$et{HandlerType} ne $$tagInfo{HandlerType});
             my $subdir = $$tagInfo{SubDirectory};
             unless ($subdir) {
                 my $nvHash = $et->GetNewValueHash($tagInfo);
@@ -1386,6 +1408,7 @@ sub WriteQuickTime($$$)
                 HasData  => $$subdir{HasData},
                 OutFile  => $outfile,
                 ChunkOffset => [ ], # (just to be safe)
+                WriteGroup => $$tagInfo{WriteGroup},
             );
             my $subTable = GetTagTable($$subdir{TagTable});
             my $newData = $et->WriteDirectory(\%subdirInfo, $subTable, $$subdir{WriteProc});
